# async process!!!!
# Manuel Amador (Rudd-O) - written on 2005-11-28
# under the GPL
from threading import Thread
from Queue import Queue,Empty
class NotReadyYet(Exception): pass
class AsyncProcess(Queue):
	"""Run a function in a daemon thread, and make its results
	available, once it's finished, in the get() and get_nowait() methods.
	
	If an exception happens in the daemon thread, the exception instance
	will be raised as part of running get() or get_nowait().  But if the
	process is still not ready when you check with get(), a NotReadyYet
	exception will be raised.
	
	Use it like this:
			
	def simulate_something_long(arg1,arg2):
		import time
		time.sleep(arg1*arg2)
	
	p = AsyncProcess(target=simulate_something_long,args=(3,4))
	p.start()
	while we_should_still_wait:
		try:
			print p.get(block=True,timeout=1)
			break
		except NotReadyYet:
			do_other_stuff()
		except Exception,e:
			print "An error occurred in the async process:",str(e)
			break
	"""
	def __init__(self,target,args):
		Queue.__init__(self)
		self.target = target
		self.args = args
	
	def start(self):
		t = Thread(target=self.__run)
		t.setDaemon(True)
		t.start()
	
	def __run(self):
		try: self.put(self.target(*self.args))
		except Exception,e: self.put(e)

	def get(self,block=True,timeout=None):
		if not hasattr(self,"retvalue"):
			try: gotten = Queue.get(self,block,timeout)
			except Empty: raise NotReadyYet
			self.retvalue = gotten
		if isinstance(self.retvalue,Exception): raise self.retvalue
		else: return self.retvalue
# end async process


__all__ = ["NotReadyYet","AsyncProcess"]
