#!/usr/bin/env python
"""
an ideal script object:
- would contain an amarokproxy instance
- would run in a loop until amarok exits
- would fork and set up signal handlers
- would contain a config object
- would contain logging handlers
- would set up everything for logging and redirect stdout/stderr

several responsibilities
holding/sving/restoring the configuration
holding the proxy
forking
all orthogonal!!!!

user uses it like this:
	
	
class SmartDJ(amaroKScript):
	logFile = file(os.path.expanduser("~/.smartdj.log")

script = SmartDJ()
script.exec_loop()



# I am currently doing:

1. defines
2. import generics
3. check for requirements
4. check for UI


#a script needs to check for requirements

"""

from amarok import amaroKProxy
import logging
from Queue import Queue
import os
import signal

logger = logging.getLogger("script interface")


class SignalQueue(Queue):
	def __init__(self):
		Queue.__init__(self)
	
	def handler(self,signal,frame):
		"""The installable signal handler"""
		logger.info( "received signal %s"%signal)
		self.put(signal)
		
	def trap(self,signal):
		from signal import signal as sigaction
		sigaction(signal,self.handler)


class amaroKScript:

	def __init__(self):
		self.proxy = amaroKProxy()
		self.signalQueue = SignalQueue()

	def exec_loop(self):
		pid = os.fork()
		if (pid): # the child, we go on as normal
			logger.debug("Parent process: forked.  PID: %s",os.getpid())
			for sig in [signal.SIGINT,signal.SIGTERM]: self.signalQueue.trap(sig)
			while True:
				try:
					sig = self.signalQueue.get(block=True,timeout=1)
					logger.debug("Parent process got signal %s, going down and letting child notice on its own",sig)
					break
				except Empty:
					childpid,status = os.waitpid(pid,os.WNOHANG)
					if not childpid: continue
					if os.WCOREDUMP(status): logger.debug("Child core dumped")
					elif os.WIFSIGNALED(status): logger.debug("Child got an unhandled signal %s",os.WTERMSIG(status))
					elif os.WIFEXITED(status): logger.debug("Child returned with return value %s",os.WEXITSTATUS(status))
					else: logger.debug("Child died with status %s",status)
			logger.debug("Parent process done")
		else:
			logger.debug("Child process: forked.  PID: %s",os.getpid())
			self.proxy.startMonitoring()